/*
Copyright (C) 2004 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package tempsecoule;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

/**
 * Calcul du temps coul depuis le big bang.
 */
public class TempsEcoule extends Applet implements ActionListener {
    
    private static final double Omega_m = 0.3;
    private Choice ch_H0;
    private TextField tf_z;
    private Label lab_t;
    
    public void init() {
        setLayout(new BorderLayout());
        Panel gauche = new Panel(new GridLayout(4, 1));
        Panel droite = new Panel(new GridLayout(4, 1));
        
        gauche.add(new Label("H0 : "));
        ch_H0 = new Choice();
        ch_H0.add("50");
        ch_H0.add("75");
        ch_H0.add("100");
        ch_H0.add("500");
        ch_H0.select("75");
        droite.add(ch_H0);
        
        gauche.add(new Label("z : "));
        tf_z = new TextField(10);
        droite.add(tf_z);
        
        gauche.add(new Label(" "));
        
        Button bok = new Button("calcul");
        bok.addActionListener(this);
        droite.add(bok);
        
        gauche.add(new Label("t : "));
        lab_t = new Label("?");
        droite.add(lab_t);
        
        add(gauche, BorderLayout.WEST);
        add(droite, BorderLayout.CENTER);
        
        validate();
        show();
    }
    
    public void actionPerformed(ActionEvent e) {
        calcul();
    }
    
    protected void calcul() {
        String sH0 = ch_H0.getSelectedItem();
        String sz = tf_z.getText();
        double H0;
        double z;
        try {
            H0 = (new Double(sH0)).doubleValue();
            z = (new Double(sz)).doubleValue();
        } catch (NumberFormatException ex) {
            System.err.println(ex.getClass().getName() + ": " + ex.getMessage());
            lab_t.setText("?");
            return;
        }
        double t = 978 * 2 / (3 * H0 * Math.sqrt(1 - Omega_m)) * asinh(Math.sqrt((1 / Omega_m - 1)/Math.pow(1 + z, 3)));
        lab_t.setText(joliFormattage(t, 3) + " milliards d'annes");
    }
    
    private double asinh(double x) {
        return(Math.log(x + Math.sqrt(x*x + 1)));
    }
    
	// mthode d'affichage de double issue du Calcotron
    public static String joliFormattage(double v, int precision) {
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else
                sv += ".10^" + exposant;
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }
}
